@@ -5,11 +5,14 @@ define the transformations and the order in which they run on the AST.
 """
 from __future__ import print_function, division, absolute_import
 
+import os
+import sys
 import ast as ast_module
 import logging
 import pprint
 import random
 import types
+import copy
 import llvm.core as lc
 
 # import numba.closures
@@ -18,19 +21,23 @@ from numba import error
 from numba import functions
 from numba import transforms
 from numba import control_flow
-from numba import optimize
 from numba import closures
 from numba import reporting
 from numba import normalize
+from numba import validate
+from numba.array_validation import ArrayValidator
+from numba.viz import cfgviz
 from numba import typesystem
 from numba.codegen import llvmwrapper
 from numba import ast_constant_folding as constant_folding
-from numba.control_flow import ssa
+from numba.control_flow import ssa, cfstats
 from numba.codegen import translate
 from numba import utils
 from numba.missing import FixMissingLocations
 from numba.type_inference import infer as type_inference
 from numba.asdl import schema
+from numba.prettyprint import (dump_ast, dump_cfg, dump_annotations,
+                               dump_llvm, dump_optimized)
 import numba.visitors
 
 from numba.specialize import comparisons
@@ -109,7 +116,7 @@ def infer_types2(env, func, restype=None, argtypes=None, **kwargs):
 
 
 def compile2(env, func, restype=None, argtypes=None, ctypes=False,
-             compile_only=False, **kwds):
+             compile_only=False, func_ast=None, **kwds):
     """
     Compile a numba annotated function.
 
@@ -122,7 +129,10 @@ def compile2(env, func, restype=None, argtypes=None, ctypes=False,
     assert 'llvm_module' not in kwds
     kwds['llvm_module'] = lc.Module.new(module_name(func))
     logger.debug(kwds)
-    func_ast = functions._get_ast(func)
+    if func_ast is None:
+        func_ast = functions._get_ast(func)
+    else:
+        func_ast = copy.deepcopy(func_ast)
     func_signature = typesystem.function(restype, argtypes)
     #pipeline, (func_signature, symtab, ast) = _infer_types2(
     #            env, func, restype, argtypes, codegen=True, **kwds)
@@ -245,6 +255,10 @@ def validate_signature(tree, env):
 
     return tree
 
+def validate_arrays(ast, env):
+    ArrayValidator(env).visit(ast)
+    return ast
+
 def update_signature(tree, env):
     func_env = env.translation.crnt
     func_signature = func_env.func_signature
@@ -302,12 +316,20 @@ def create_lfunc3(tree, env):
     create_lfunc(tree, env)
     return tree
 
+# ______________________________________________________________________
+
+class ValidateASTStage(PipelineStage):
+    def transform(self, ast, env):
+        validate.ValidateAST().visit(ast)
+        return ast
 
 class NormalizeASTStage(PipelineStage):
     def transform(self, ast, env):
         transform = self.make_specializer(normalize.NormalizeAST, ast, env)
         return transform.visit(ast)
 
+# ______________________________________________________________________
+
 class ControlFlowAnalysis(PipelineStage):
     _pre_condition_schema = None
 
@@ -327,16 +349,9 @@ class ControlFlowAnalysis(PipelineStage):
         ast = transform.visit(ast)
         env.translation.crnt.symtab = transform.symtab
         ast.flow = transform.flow
-        env.translation.crnt.ast.cfg_transform = transform
         return ast
 
 
-def dump_cfg(ast, env):
-    if env.translation.crnt.cfg_transform.graphviz:
-        env.translation.crnt.cfg_transform.render_gv(ast)
-    return ast
-
-
 class ConstFolding(PipelineStage):
     def check_preconditions(self, ast, env):
         assert not hasattr(env.crnt, 'constvars')
@@ -397,6 +412,51 @@ class TransformFor(PipelineStage):
                                           env)
         return transform.visit(ast)
 
+class TransformBuiltinLoops(PipelineStage):
+    def transform(self, ast, env):
+        transform = self.make_specializer(loops.TransformBuiltinLoops, ast,
+                                          env)
+        return transform.visit(ast)
+
+#----------------------------------------------------------------------------
+# Prange
+#----------------------------------------------------------------------------
+
+def run_prange(name):
+    def wrapper(ast, env):
+        from numba import parallel
+        stage = getattr(parallel, name)
+        return PipelineStage().make_specializer(stage, ast, env).visit(ast)
+
+    wrapper.__name__ = name
+    return wrapper
+
+ExpandPrange = run_prange('PrangeExpander')
+RewritePrangePrivates = run_prange('PrangePrivatesReplacer')
+CleanupPrange = run_prange('PrangeCleanup')
+
+
+class UpdateAttributeStatements(PipelineStage):
+    def transform(self, ast, env):
+        func_env = env.translation.crnt
+
+        for block in func_env.flow.blocks:
+            stats = []
+            for cf_stat in block.stats:
+                if (isinstance(cf_stat, cfstats.AttributeAssignment) and
+                        isinstance(cf_stat.lhs, ast_module.Attribute)):
+                    value = cf_stat.lhs.value
+                    if (isinstance(value, ast_module.Name) and
+                                value.id in func_env.kill_attribute_assignments):
+                        cf_stat = None
+
+                if cf_stat:
+                    stats.append(cf_stat)
+
+            block.stats = stats
+
+        return ast
+
 #----------------------------------------------------------------------------
 # Specializing/Lowering Transforms
 #----------------------------------------------------------------------------
@@ -426,27 +486,21 @@ class SpecializeSSA(PipelineStage):
         return ast
 
 class SpecializeClosures(SimplePipelineStage):
-
     transformer = closures.ClosureSpecializer
 
-
 class Optimize(PipelineStage):
     def transform(self, ast, env):
         return ast
 
-
-class Preloader(PipelineStage):
-    def transform(self, ast, env):
-        transform = self.make_specializer(optimize.Preloader, ast, env)
-        return transform.visit(ast)
-
-
 class SpecializeLoops(PipelineStage):
     def transform(self, ast, env):
         transform = self.make_specializer(loops.SpecializeObjectIteration, ast,
                                           env)
         return transform.visit(ast)
 
+class LowerRaise(PipelineStage):
+    def transform(self, ast, env):
+        return self.make_specializer(exceptions.LowerRaise, ast, env).visit(ast)
 
 class LateSpecializer(PipelineStage):
     def transform(self, ast, env):
@@ -484,11 +538,11 @@ def cleanup_symtab(ast, env):
 
 class FixASTLocations(PipelineStage):
     def transform(self, ast, env):
-        fixer = self.make_specializer(FixMissingLocations, ast, env)
-        fixer.visit(ast)
+        lineno = getattr(ast, 'lineno', 1)
+        col_offset = getattr(ast, 'col_offset', 1)
+        FixMissingLocations(lineno, col_offset).visit(ast)
         return ast
 
-
 class CodeGen(PipelineStage):
     def transform(self, ast, env):
         func_env = env.translation.crnt
