@@ -168,7 +168,7 @@ class _NumbaType(Type):
         a 2D strided array of doubles. The syntax is the same as for
         Cython memoryviews.
         """
-        assert isinstance(item, (tuple, slice))
+        assert isinstance(item, (tuple, slice)), item
 
         def verify_slice(s):
             if s.start or s.stop or s.step not in (None, 1):
@@ -375,14 +375,14 @@ class sized_pointer(NumbaType):
     argnames = ["base_type", "size"]
     flags = ["pointer"]
 
-    def __eq__(self, other):
-        if other.is_sized_pointer:
-            return (self.base_type == other.base_type and
-                    self.size == other.size)
-        return other.is_pointer and self.base_type == other.base_type
-
-    def __hash__(self):
-        return hash(self.base_type.pointer())
+    # def __eq__(self, other):
+    #     if other.is_sized_pointer:
+    #         return (self.base_type == other.base_type and
+    #                 self.size == other.size)
+    #     return other.is_pointer and self.base_type == other.base_type
+    #
+    # def __hash__(self):
+    #     return hash(self.base_type.pointer())
 
 @consing
 class carray(NumbaType):
@@ -568,7 +568,7 @@ class ContainerListType(NumbaType):
     """
 
     argnames = ["base_type", "size"]
-    flags = ["object"]
+    flags = ["object", "container"]
 
     def is_sized(self):
         return self.size >= 0
